@@ -12,6 +12,7 @@ import random
 import numpy
 import time
 import re
+from typing import List, Dict, Tuple, Union
 from transformers import GPT2Model, GPT2LMHeadModel, GPT2Config
 from benchmark_helper import Precision
 
@@ -52,23 +53,64 @@ class MyGPT2LMHeadModel(GPT2LMHeadModel):
         return super().forward(input_ids, position_ids=position_ids, attention_mask=attention_mask, past=past)
 
 
-# Maps model class name to a tuple of model class and name of first output
-MODEL_CLASSES = {'GPT2LMHeadModel': (MyGPT2LMHeadModel, 'logits'), 'GPT2Model': (MyGPT2Model, 'last_state')}
+class MyGPT2LMHeadModel_NoPadding(GPT2LMHeadModel):
+    """ Here we wrap a class for Onnx model conversion for GPT2LMHeadModel with past state and no padding.
+        When you always use batch_size=1 in inference, there is no padding in inputs. In such case, position_ids
+        and attention_mask need no be in inputs.
+    """
+    def __init__(self, config):
+        super().__init__(config)
+
+    def forward(self, input_ids, *past):
+        return super().forward(input_ids, past=past)
+
+
+# Maps model class name to a tuple of model class, name of first output and use padding or not
+MODEL_CLASSES = {
+    'GPT2LMHeadModel': (MyGPT2LMHeadModel, 'logits', True),
+    'GPT2LMHeadModel_NoPadding': (MyGPT2LMHeadModel_NoPadding, 'logits', False),
+    'GPT2Model': (MyGPT2Model, 'last_state', True),
+}
+
+
+class Gpt2Inputs:
+    def __init__(self, input_ids, position_ids, attention_mask, past):
+        self.input_ids: torch.LongTensor = input_ids
+        self.position_ids: torch.LongTensor = position_ids
+        self.attention_mask: Union[torch.FloatTensor, torch.HalfTensor] = attention_mask
+        self.past: Union[List[torch.FloatTensor], List[torch.HalfTensor]] = past
+
+    def to_list(self) -> List:
+        input_list = [v for v in [self.input_ids, self.position_ids, self.attention_mask] if v is not None]
+        if self.past:
+            input_list.extend(self.past)
+
+        return input_list
+
+    def to_tuple(self) -> Tuple:
+        return tuple(v for v in [self.input_ids, self.position_ids, self.attention_mask, self.past] if v is not None)
+
+    def to_fp32(self):
+        attention_mask = self.attention_mask.to(dtype=torch.float32) if self.attention_mask is not None else None
+        past = [p.to(dtype=torch.float32) for p in self.past]
+        return Gpt2Inputs(self.input_ids, self.position_ids, attention_mask, past)
 
 
 class Gpt2Helper:
     """ A helper class for Gpt2 model conversion, inference and verification.
     """
     @staticmethod
-    def get_dummy_inputs(batch_size,
-                         past_sequence_length,
-                         sequence_length,
-                         num_attention_heads,
-                         hidden_size,
-                         num_layer,
-                         vocab_size,
-                         device,
-                         float16=False):
+    def get_dummy_inputs(batch_size: int,
+                         past_sequence_length: int,
+                         sequence_length: int,
+                         num_attention_heads: int,
+                         hidden_size: int,
+                         num_layer: int,
+                         vocab_size: int,
+                         device: torch.device,
+                         float16: bool = False,
+                         has_position_ids: bool = True,
+                         has_attention_mask: bool = True) -> Gpt2Inputs:
         """ Create random inputs for GPT2 model.
         Returns torch tensors of input_ids, position_ids, attention_mask and a list of past state tensors.
         """
@@ -82,21 +124,29 @@ class Gpt2Helper:
                                   dtype=torch.int64,
                                   device=device)
 
-        total_sequence_length = past_sequence_length + sequence_length
-        attention_mask = torch.ones([batch_size, total_sequence_length], dtype=float_type, device=device)
-        if total_sequence_length >= 2:
-            padding_position = random.randint(0, total_sequence_length - 1)  # test input with padding.
-            attention_mask[:, padding_position] = 0
+        attention_mask = None
+        if has_attention_mask:
+            total_sequence_length = past_sequence_length + sequence_length
+            attention_mask = torch.ones([batch_size, total_sequence_length], dtype=float_type, device=device)
+            if total_sequence_length >= 2:
+                padding_position = random.randint(0, total_sequence_length - 1)  # test input with padding.
+                attention_mask[:, padding_position] = 0
 
         # Deduce position_ids from attention mask
-        position_ids = (attention_mask.long().cumsum(-1) - 1)
-        position_ids.masked_fill_(position_ids < 0, 0)
-        position_ids = position_ids[:, past_sequence_length:]
+        position_ids = None
+        if has_position_ids:
+            position_ids = (attention_mask.long().cumsum(-1) - 1)
+            position_ids.masked_fill_(position_ids < 0, 0)
+            position_ids = position_ids[:, past_sequence_length:]
 
-        return input_ids, position_ids, attention_mask, past
+        return Gpt2Inputs(input_ids, position_ids, attention_mask, past)
 
     @staticmethod
-    def get_output_shapes(batch_size, past_sequence_length, sequence_length, config, model_class="GPT2LMHeadModel"):
+    def get_output_shapes(batch_size: int,
+                          past_sequence_length: int,
+                          sequence_length: int,
+                          config: GPT2Config,
+                          model_class: str = "GPT2LMHeadModel") -> Dict[str, List[int]]:
         """ Returns a dictionary with output name as key, and shape as value.
         """
         num_attention_heads = config.num_attention_heads
@@ -106,9 +156,7 @@ class Gpt2Helper:
 
         output_name = MODEL_CLASSES[model_class][1]
 
-        last_state_shape = [
-            batch_size, sequence_length, vocab_size if model_class == "GPT2LMHeadModel" else hidden_size
-        ]
+        last_state_shape = [batch_size, sequence_length, vocab_size if output_name == "logits" else hidden_size]
         present_state_shape = [
             2, batch_size, num_attention_heads, past_sequence_length + sequence_length,
             int(hidden_size / num_attention_heads)
@@ -176,8 +224,14 @@ class Gpt2Helper:
         return is_all_close
 
     @staticmethod
-    def export_onnx(model, device, onnx_model_path, verbose=False, use_external_data_format=False):
-        """ Export GPT-2 model with past state to ONNX model
+    def export_onnx(model,
+                    device,
+                    onnx_model_path: str,
+                    verbose: bool = False,
+                    use_external_data_format: bool = False,
+                    has_position_ids: bool = True,
+                    has_attention_mask: bool = True):
+        """ Export GPT-2 model with past state to ONNX model.
         """
         config: GPT2Config = model.config
         num_layer = config.n_layer
@@ -189,11 +243,11 @@ class Gpt2Helper:
                                                    num_layer=num_layer,
                                                    vocab_size=config.vocab_size,
                                                    device=device,
-                                                   float16=False)
+                                                   float16=False,
+                                                   has_position_ids=has_position_ids,
+                                                   has_attention_mask=has_attention_mask)
+        input_list = dummy_inputs.to_list()
 
-        dummy_input_ids, dummy_position_ids, dummy_attention_mask, dummy_past = dummy_inputs
-
-        input_list = [dummy_input_ids, dummy_position_ids, dummy_attention_mask] + dummy_past
         with torch.no_grad():
             outputs = model(*input_list)
 
@@ -218,17 +272,23 @@ class Gpt2Helper:
         for name in present_names:
             dynamic_axes[name] = {1: 'batch_size', 3: 'total_seq_len'}
 
-        dynamic_axes['attention_mask'] = {0: 'batch_size', 1: 'total_seq_len'}
-        dynamic_axes['position_ids'] = {0: 'batch_size', 1: 'seq_len'}
+        input_names = ['input_ids']
+        if has_position_ids:
+            dynamic_axes['position_ids'] = {0: 'batch_size', 1: 'seq_len'}
+            input_names.append('position_ids')
+        if has_attention_mask:
+            dynamic_axes['attention_mask'] = {0: 'batch_size', 1: 'total_seq_len'}
+            input_names.append('attention_mask')
+        input_names.extend(past_names)
 
         logger.info(
-            f"Shapes: input_ids={dummy_input_ids.shape} past={dummy_past[0].shape} output={outputs[0].shape} present={outputs[1][0].shape}"
+            f"Shapes: input_ids={dummy_inputs.input_ids.shape} past={dummy_inputs.past[0].shape} output={outputs[0].shape} present={outputs[1][0].shape}"
         )
 
         torch.onnx.export(model,
                           args=tuple(input_list),
                           f=onnx_model_path,
-                          input_names=['input_ids', 'position_ids', 'attention_mask'] + past_names,
+                          input_names=input_names,
                           output_names=output_names,
                           example_outputs=outputs,
                           dynamic_axes=dynamic_axes,
@@ -237,6 +297,7 @@ class Gpt2Helper:
                           use_external_data_format=use_external_data_format,
                           verbose=verbose)
 
+    @staticmethod
     def optimize_onnx(onnx_model_path,
                       optimized_model_path,
                       is_float16,
@@ -259,18 +320,13 @@ class Gpt2Helper:
         m.save_model_to_file(optimized_model_path, use_external_data_format)
 
     @staticmethod
-    def pytorch_inference(model, inputs, total_runs=0):
+    def pytorch_inference(model, inputs: Gpt2Inputs, total_runs: int = 0):
         """ Run inference of PyTorch model, and returns average latency in ms when total_runs > 0 besides outputs.
         """
-        logger.debug(f"start pytorch_inference")
-        input_ids, position_ids, attention_mask, past = inputs
+        logger.debug("start pytorch_inference")
 
-        # Convert it back to fp32 as the PyTroch model cannot deal with half input.
-        if attention_mask is not None:
-            attention_mask = attention_mask.to(dtype=torch.float32)
-        past = [p.to(dtype=torch.float32) for p in past]
-
-        input_list = [input_ids, position_ids, attention_mask] + past
+        # Convert it to fp32 as the PyTroch model cannot deal with half input.
+        input_list = inputs.to_fp32().to_list()
 
         with torch.no_grad():
             outputs = model(*input_list)
@@ -291,23 +347,22 @@ class Gpt2Helper:
         return outputs, average_latency
 
     @staticmethod
-    def onnxruntime_inference(ort_session, inputs, total_runs=0):
+    def onnxruntime_inference(ort_session, inputs: Gpt2Inputs, total_runs: int = 0):
         """ Run inference of ONNX model, and returns average latency in ms when total_runs > 0 besides outputs.
         """
         logger.debug(f"start onnxruntime_inference")
-        input_ids, position_ids, attention_mask, past = inputs
 
-        ort_inputs = {'input_ids': numpy.ascontiguousarray(input_ids.cpu().numpy())}
+        ort_inputs = {'input_ids': numpy.ascontiguousarray(inputs.input_ids.cpu().numpy())}
 
-        if past is not None:
-            for i, past_i in enumerate(past):
+        if inputs.past is not None:
+            for i, past_i in enumerate(inputs.past):
                 ort_inputs[f'past_{i}'] = numpy.ascontiguousarray(past_i.cpu().numpy())
 
-        if attention_mask is not None:
-            ort_inputs['attention_mask'] = numpy.ascontiguousarray(attention_mask.cpu().numpy())
+        if inputs.attention_mask is not None:
+            ort_inputs['attention_mask'] = numpy.ascontiguousarray(inputs.attention_mask.cpu().numpy())
 
-        if position_ids is not None:
-            ort_inputs['position_ids'] = numpy.ascontiguousarray(position_ids.cpu().numpy())
+        if inputs.position_ids is not None:
+            ort_inputs['position_ids'] = numpy.ascontiguousarray(inputs.position_ids.cpu().numpy())
 
         ort_outputs = ort_session.run(None, ort_inputs)
         if total_runs == 0:
@@ -384,20 +439,19 @@ class Gpt2Helper:
 
     @staticmethod
     def onnxruntime_inference_with_binded_io(ort_session,
-                                             inputs,
-                                             output_buffers,
-                                             output_shapes,
-                                             total_runs=0,
-                                             return_numpy=True,
-                                             include_copy_output_latency=False):
+                                             inputs: Gpt2Inputs,
+                                             output_buffers: Dict[str, torch.Tensor],
+                                             output_shapes: Dict[str, List[int]],
+                                             total_runs: int = 0,
+                                             return_numpy: bool = True,
+                                             include_copy_output_latency: bool = False):
         """ Inference with IO binding. Returns outputs, and optional latency when total_runs > 0.
         """
         logger.debug(f"start onnxruntime_inference_with_binded_io")
-        input_ids, position_ids, attention_mask, past = inputs
 
         # Bind inputs and outputs to onnxruntime session
-        io_binding = Gpt2Helper.prepare_io_binding(ort_session, input_ids, position_ids, attention_mask, past,
-                                                   output_buffers, output_shapes)
+        io_binding = Gpt2Helper.prepare_io_binding(ort_session, inputs.input_ids, inputs.position_ids,
+                                                   inputs.attention_mask, inputs.past, output_buffers, output_shapes)
 
         # Run onnxruntime with io binding
         ort_session.run_with_iobinding(io_binding)
@@ -433,7 +487,9 @@ class Gpt2Helper:
                     atol=5e-4,
                     total_test_cases=100,
                     use_io_binding=True,
-                    model_class="GPT2LMHeadModel"):
+                    model_class="GPT2LMHeadModel",
+                    has_position_ids=True,
+                    has_attention_mask=True):
         """ Generate random inputs and compare the results of PyTorch and Onnx Runtime.
         """
 
@@ -463,7 +519,9 @@ class Gpt2Helper:
                 f"Running parity test for batch_size={batch_size} past_sequence_length={past_sequence_length}...")
             dummy_inputs = Gpt2Helper.get_dummy_inputs(batch_size, past_sequence_length, sequence_length,
                                                        config.num_attention_heads, config.hidden_size, config.n_layer,
-                                                       config.vocab_size, device, is_float16)
+                                                       config.vocab_size, device, is_float16, has_position_ids,
+                                                       has_attention_mask)
+
             outputs = Gpt2Helper.pytorch_inference(model, dummy_inputs)
             if use_io_binding:
                 ort_outputs = Gpt2Helper.onnxruntime_inference(ort_session, dummy_inputs)
@@ -482,20 +540,21 @@ class Gpt2Helper:
         return passed_test_cases == total_test_cases
 
     @staticmethod
-    def torchscript(model, config, device):
+    def torchscript(model, config, device, has_position_ids=True, has_attention_mask=True):
         """ JIT trace for TorchScript.
         """
-        dummy_inputs = Gpt2Helper.get_dummy_inputs(batch_size=1,
-                                                   past_sequence_length=1,
-                                                   sequence_length=1,
-                                                   num_attention_heads=config.num_attention_heads,
-                                                   hidden_size=config.hidden_size,
-                                                   num_layer=config.n_layer,
-                                                   vocab_size=config.vocab_size,
-                                                   device=device,
-                                                   float16=False)
-        dummy_input_ids, dummy_position_ids, dummy_attention_mask, dummy_past = dummy_inputs
-        return torch.jit.trace(model, [dummy_input_ids, dummy_position_ids, dummy_attention_mask] + dummy_past)
+        input_list = Gpt2Helper.get_dummy_inputs(batch_size=1,
+                                                 past_sequence_length=1,
+                                                 sequence_length=1,
+                                                 num_attention_heads=config.num_attention_heads,
+                                                 hidden_size=config.hidden_size,
+                                                 num_layer=config.n_layer,
+                                                 vocab_size=config.vocab_size,
+                                                 device=device,
+                                                 float16=False,
+                                                 has_position_ids=has_position_ids,
+                                                 has_attention_mask=has_attention_mask).to_list()
+        return torch.jit.trace(model, input_list)
 
     @staticmethod
     def get_onnx_paths(output_dir,
@@ -505,8 +564,11 @@ class Gpt2Helper:
                        new_folder=False):
         """ Build a  path name for given model based on given attributes.
         """
-        model_name = model_name_or_path if re.match('^[\w_-]+$',
-                                                    model_name_or_path) else os.path.dirname(model_name_or_path)
+        model_name = model_name_or_path
+        if not re.match('^[\w_-]+$', model_name_or_path):  # It is not a name, shall be a path
+            assert os.path.isdir(model_name_or_path)
+            from pathlib import Path
+            model_name = Path(model_name_or_path).parts[-1]
 
         if model_class != 'GPT2LMHeadModel':
             model_name += "_" + model_class
